/* AP_AetherHarp-4_V12.
   Alex Pikkert April 2023.
   A music instrument with 8 notes in 6 octaves, played by moving your hands on a low (5 cm) or high (15 cm) level above 8 sensors.
   The tones are sent to a computer as digital codes, resulting in real music via the MIDI protocol, using 128 different instruments.
   The instrument is based on the Aetherharp design with a Teensy 3.5 computer board published by XenonJohn on the Instructables website:
   https://www.instructables.com/Aetherharp-Three-Level-24-Note-MIDI-Air-harp/
   (This design is updated from "Air Piano V12" (from John Dingley, July 2021).
   My first update (Aetherharp-2) can be found here: https://www.instructables.com/Aetherharp-2/
   My second update (Aetherharp-3) can be found here:https://www.instructables.com/Aetherharp-3/
   This update (Aetherharp-4) can be found here: https://www.instructables.com/Aetherharp-4-MIDI-Air-Piano/
   This Aetherharp V4 has the following changes and additions compared to the original designs:
  .An LCD display with a rotary encoder for selecting MIDI instruments from a list.
  .8 notes in 6 octaves.
  .Two switches to toggle ON/OFF vibrato and sustain effects.
  .Two pushbuttons to select (UP/DOWN) 5 octaves, starting with the E4 octave. The 6th (E1) octave can only be reached with the red notes combined with octave --.
  .Notes can be programmed by activating the rotary encoder and its pushbutton.
  .Notes can be reset to the original set by pushing the select UP button during note programming.
  .Two Light Dependent Resistors (LDR) to allow pitch bending while playing by moving your hand above them.
  .The Sharp IR sensors (type GP2YA41SK0F) are turned 90 degrees for better reaction to hand movements. (see the Sharp datasheet for details).
  .The SharpIR Arduino library is not used, the sensor reading is done via the standard analogRead Arduino instruction.
  .Outlier values are compensated via RunningMedian, these random notes seem to occur only when no hand is detected.
  .An external 5VDC power supply for the sensors is wired per sensor to a central connection with a large capacitor to avoid power noise.
  .Indication LED for the 5VDC external power supply.
  .The sensors are mounted directly underneath the acrylic cover to avoid blocking of the IR sensor light.
  .Bi-color indication LEDs for each sensor.(Blue for high level note and red for low level note).
   These leds are ON as long as a note is played and when the sustain setting is active.
  .The analog ground (AGND)is used for the Sharp analog sensors, all other digital connections use the normal GND connection.
*/

//import libraries in the sketch

#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 15, 2);
int RunningMedianNumber = 10;

#include <RunningMedian.h>
RunningMedian Samples7 = RunningMedian(RunningMedianNumber);
RunningMedian Samples6 = RunningMedian(RunningMedianNumber);
RunningMedian Samples5 = RunningMedian(RunningMedianNumber);
RunningMedian Samples4 = RunningMedian(RunningMedianNumber);
RunningMedian Samples3 = RunningMedian(RunningMedianNumber);
RunningMedian Samples2 = RunningMedian(RunningMedianNumber);
RunningMedian Samples1 = RunningMedian(RunningMedianNumber);
RunningMedian Samples0 = RunningMedian(RunningMedianNumber);

int Note[]        = {64, 66, 68, 69, 71, 73, 75, 76};
int AddNote       = 12;
bool NotePlayed[] = {false, false, false, false, false, false, false, false};
bool TrigAddon[]  = {false, false, false, false, false, false, false, false};
int SensorValue[] = {0, 0, 0, 0, 0, 0, 0, 0};
int SensorFinal[] = {0, 0, 0, 0, 0, 0, 0, 0};
int SensorNumber = 8;
int BlueLed[] = {7, 9, 11, 24, 26, 28, 30, 32};
int RedLed[] = {6, 8, 10, 12, 25, 27, 29, 31};
int Sensor[] = {A14, A15, A16, A17, A18, A19, A20, A0} ;
const int PitchBendPin = A6;
const byte upEncoderPin = 1;
const byte downEncoderPin = 2;
const byte NoteSelectButton = 3;
const byte SustainSwitch = 4;
const byte VibratoSwitch = 5;
const byte OctaveDownButton = 22;
const byte OctaveUpButton = 23;
int Thresh = 180;
int ThreshHigh = 220;
int Wait = 3;
int Octave = 2;
const char* OctString[5] = {"--", "- ", "  ", "+ ", "++",};
bool SustainOn = false;
int j = 0; // counter
int k = 0; // counter
int m = 0; // counter
int PitchBendSample;
int PitchValue;
int LowerTrigger;
int HigherTrigger;
int MidPoint;
bool Check = false;

void setup() {
  //Enable serial communication for testing.
  //Serial.begin(115200);

  //define the input pins with the built-in pullup resistors for the buttons & switches
  pinMode(1, INPUT_PULLUP);  // midi instrument selection clockwise
  pinMode(2, INPUT_PULLUP);  // midi instrument selection counterclockwise
  pinMode(3, INPUT_PULLUP);  // rotary encoder button
  pinMode(4, INPUT_PULLUP);  // midi select vibrato ON/OFF
  pinMode(5, INPUT_PULLUP);  // midi select sustain ON/OFF
  pinMode(22, INPUT_PULLUP); // midi select octave DOWN
  pinMode(23, INPUT_PULLUP); // midi select octave UP
  pinMode(A6, INPUT);        // pitchbend input

  // define the output pins for 16 blue an red indication LEDs
  pinMode(6, OUTPUT);
  pinMode(7, OUTPUT);
  pinMode(8, OUTPUT);
  pinMode(9, OUTPUT);
  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(24, OUTPUT);
  pinMode(25, OUTPUT);
  pinMode(26, OUTPUT);
  pinMode(27, OUTPUT);
  pinMode(28, OUTPUT);
  pinMode(29, OUTPUT);
  pinMode(30, OUTPUT);
  pinMode(31, OUTPUT);
  pinMode(32, OUTPUT);

  // Led test
  for (j = 0; j < SensorNumber; j++) {
    digitalWrite(BlueLed[j], HIGH);
    delay(300);
    digitalWrite(BlueLed[j], LOW);
  }
  delay(300);
  for (j = SensorNumber - 1; j >= 0; j--) {
    digitalWrite(RedLed[j], HIGH);
    delay(300);
    digitalWrite(RedLed[j], LOW);
  }
  delay(300);

  //setup for the start of the LCD display
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0); lcd.print(" *AETHERHARP-4* ");
  lcd.setCursor(0, 1); lcd.print("      *V12*     ");
  delay(3000);
  lcd.setCursor(0, 0); lcd.print(" * start VMPK * ");
  lcd.setCursor(0, 1); lcd.print("                ");
  delay(3000);
  lcd.setCursor(0, 0); lcd.print("     oct        ");
  lcd.setCursor(0, 1); lcd.print("*set instrument*");

  //sets up an initial dead zone in the pitchbend LDRs either way of midpoint before a note is bent
  MidPoint = analogRead(PitchBendPin);
  HigherTrigger = MidPoint + 50;
  LowerTrigger = MidPoint - 50;
}

void loop() {
  //keep an eye on the switches and the pitchbend LDRs via subroutines
  CheckOctave();
  CheckSustain();
  CheckVibrato();
  CheckInstrument();
  CheckPitchBend();
  CheckNote();

  // ********************sensor 7*******************
  if (NotePlayed[7] == false) {
    SensorValue[7] = analogRead(Sensor[7]);
    Samples7.add(SensorValue[7]);
    delay(Wait);
    SensorFinal[7] = Samples7.getMedian();
    if (SensorFinal[7] > ThreshHigh) {
      Note[7] = Note[7] - AddNote;
      TrigAddon[7] = true;
    }
    if (SensorFinal[7] > Thresh) {
      usbMIDI.sendNoteOn(Note[7], 255, 1);
      NotePlayed[7] = true;
      if (TrigAddon[7] == true) {
        digitalWrite(RedLed[7], HIGH);
      }
      else {
        digitalWrite(BlueLed[7], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[7] == true) {
      SensorValue[7] = analogRead(Sensor[7]);
      Samples7.add(SensorValue[7]);
      delay(Wait);
      SensorFinal[7] = Samples7.getMedian();
      if (SensorFinal[7] < Thresh) {
        usbMIDI.sendNoteOff(Note[7], 0, 1);
        usbMIDI.sendNoteOff(Note[7] - AddNote, 0, 1);
        NotePlayed[7] = false;
      }
      if (TrigAddon[7] == true) {
        TrigAddon[7] = false;
        Note[7] = Note[7] + AddNote;
      }
    }
  }

  if (SustainOn == false && NotePlayed[7] == false)
  { digitalWrite(RedLed[7], LOW);
    digitalWrite(BlueLed[7], LOW);
  }

  // ********************sensor 6*******************
  if (NotePlayed[6] == false) {
    SensorValue[6] = analogRead(Sensor[6]);
    Samples6.add(SensorValue[6]);
    delay(Wait);
    SensorFinal[6] = Samples6.getMedian();
    if (SensorFinal[6] > ThreshHigh) {
      Note[6] = Note[6] - AddNote;
      TrigAddon[6] = true;
    }
    if (SensorFinal[6] > Thresh) {
      usbMIDI.sendNoteOn(Note[6], 255, 1);
      NotePlayed[6] = true;
      if (TrigAddon[6] == true) {
        digitalWrite(RedLed[6], HIGH);
      }
      else {
        digitalWrite(BlueLed[6], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[6] == true) {
      SensorValue[6] = analogRead(Sensor[6]);
      Samples6.add(SensorValue[6]);
      delay(Wait);
      SensorFinal[6] = Samples6.getMedian();
      if (SensorFinal[6] < Thresh) {
        usbMIDI.sendNoteOff(Note[6], 0, 1);
        usbMIDI.sendNoteOff(Note[6] - AddNote, 0, 1);
        NotePlayed[6] = false;
      }
      if (TrigAddon[6] == true) {
        TrigAddon[6] = false;
        Note[6] = Note[6] + AddNote;
      }
    }
  }
  if (SustainOn == false && NotePlayed[6]  == false) {
    digitalWrite(RedLed[6], LOW);
    digitalWrite(BlueLed[6], LOW);
  }

  // ********************sensor 5*******************
  if (NotePlayed[5] == false) {
    SensorValue[5] = analogRead(Sensor[5]);
    Samples5.add(SensorValue[5]);
    delay(Wait);
    SensorFinal[5] = Samples5.getMedian();
    if (SensorFinal[5] > ThreshHigh) {
      Note[5] = Note[5] - AddNote;
      TrigAddon[5] = true;
    }
    if (SensorFinal[5] > Thresh) {
      usbMIDI.sendNoteOn(Note[5], 255, 1);
      NotePlayed[5] = true;
      if (TrigAddon[5] == true) {
        digitalWrite(RedLed[5], HIGH);
      }
      else {
        digitalWrite(BlueLed[5], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[5] == true) {
      SensorValue[5] = analogRead(Sensor[5]);
      Samples5.add(SensorValue[5]);
      delay(Wait);
      SensorFinal[5] = Samples5.getMedian();
      if (SensorFinal[5] < Thresh) {
        usbMIDI.sendNoteOff(Note[5], 0, 1);
        usbMIDI.sendNoteOff(Note[5] - AddNote, 0, 1);
        NotePlayed[5] = false;        
      }
      if (TrigAddon[5] == true) {
        TrigAddon[5] = false;
        Note[5] = Note[5] + AddNote;
      }
    }
  }
  if (SustainOn == false && NotePlayed[5]  == false) {
    digitalWrite(RedLed[5], LOW);
    digitalWrite(BlueLed[5], LOW);
  }

  // ********************sensor 4*******************
  if (NotePlayed[4] == false) {
    SensorValue[4] = analogRead(Sensor[4]);
    Samples4.add(SensorValue[4]);
    delay(Wait);
    SensorFinal[4] = Samples4.getMedian();
    if (SensorFinal[4] > ThreshHigh) {
      Note[4] = Note[4] - AddNote;
      TrigAddon[4] = true;
    }
    if (SensorFinal[4] > Thresh) {
      usbMIDI.sendNoteOn(Note[4], 255, 1);
      NotePlayed[4] = true;
      if (TrigAddon[4] == true) {
        digitalWrite(RedLed[4], HIGH);
      }
      else {
        digitalWrite(BlueLed[4], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[4] == true) {
      SensorValue[4] = analogRead(Sensor[4]);
      Samples4.add(SensorValue[4]);
      delay(Wait);
      SensorFinal[4] = Samples4.getMedian();
      if (SensorFinal[4] < Thresh) {
        usbMIDI.sendNoteOff(Note[4], 0, 1);
        usbMIDI.sendNoteOff(Note[4] - AddNote, 0, 1);
        NotePlayed[4] = false;        
      }
      if (TrigAddon[4] == true) {
        TrigAddon[4] = false;
        Note[4] = Note[4] + AddNote;
      }
    }
  }
  if (SustainOn == false && NotePlayed[4]  == false) {
    digitalWrite(RedLed[4], LOW);
    digitalWrite(BlueLed[4], LOW);
  }

  // ********************sensor 3*******************
  if (NotePlayed[3] == false) {
    SensorValue[3] = analogRead(Sensor[3]);
    Samples3.add(SensorValue[3]);
    delay(Wait);
    SensorFinal[3] = Samples3.getMedian();
    if (SensorFinal[3] > ThreshHigh) {
      Note[3] = Note[3] - AddNote;
      TrigAddon[3] = true;
    }
    if (SensorFinal[3] > Thresh) {
      usbMIDI.sendNoteOn(Note[3], 255, 1);
      NotePlayed[3] = true;
      if (TrigAddon[3] == true) {
        digitalWrite(RedLed[3], HIGH);
      }
      else {
        digitalWrite(BlueLed[3], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[3] == true) {
      SensorValue[3] = analogRead(Sensor[3]);
      Samples3.add(SensorValue[3]);
      delay(Wait);
      SensorFinal[3] = Samples3.getMedian();
      if (SensorFinal[3] < Thresh) {
        usbMIDI.sendNoteOff(Note[3], 0, 1);
        usbMIDI.sendNoteOff(Note[3] - AddNote, 0, 1);
        NotePlayed[3] = false;        
      }
      if (TrigAddon[3] == true) {
        TrigAddon[3] = false;
        Note[3] = Note[3] + AddNote;
      }
    }
  }
  if (SustainOn == false && NotePlayed[3]  == false) {
    digitalWrite(RedLed[3], LOW);
    digitalWrite(BlueLed[3], LOW);
  }

  // ********************sensor 2*******************
  if (NotePlayed[2] == false) {
    SensorValue[2] = analogRead(Sensor[2]);
    Samples2.add(SensorValue[2]);
    delay(Wait);
    SensorFinal[2] = Samples2.getMedian();
    if (SensorFinal[2] > ThreshHigh) {
      Note[2] = Note[2] - AddNote;
      TrigAddon[2] = true;
    }
    if (SensorFinal[2] > Thresh) {
      usbMIDI.sendNoteOn(Note[2], 255, 1);
      NotePlayed[2] = true;
      if (TrigAddon[2] == true) {
        digitalWrite(RedLed[2], HIGH);
      }
      else {
        digitalWrite(BlueLed[2], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[2] == true) {
      SensorValue[2] = analogRead(Sensor[2]);
      Samples2.add(SensorValue[2]);
      delay(Wait);
      SensorFinal[2] = Samples2.getMedian();
      if (SensorFinal[2] < Thresh) {
        usbMIDI.sendNoteOff(Note[2], 0, 1);
        usbMIDI.sendNoteOff(Note[2] - AddNote, 0, 1);
        NotePlayed[2] = false;        
      }
      if (TrigAddon[2] == true) {
        TrigAddon[2] = false;
        Note[2] = Note[2] + AddNote;
      }
    }
  }
  if (SustainOn == false && NotePlayed[2]  == false) {
    digitalWrite(RedLed[2], LOW);
    digitalWrite(BlueLed[2], LOW);
  }

  // ********************sensor 1*******************
  if (NotePlayed[1] == false) {
    SensorValue[1] = analogRead(Sensor[1]);
    Samples1.add(SensorValue[1]);
    delay(Wait);
    SensorFinal[1] = Samples1.getMedian();
    if (SensorFinal[1] > ThreshHigh) {
      Note[1] = Note[1] - AddNote;
      TrigAddon[1] = true;
    }
    if (SensorFinal[1] > Thresh) {
      usbMIDI.sendNoteOn(Note[1], 255, 1);
      NotePlayed[1] = true;
      if (TrigAddon[1] == true) {
        digitalWrite(RedLed[1], HIGH);
      }
      else {
        digitalWrite(BlueLed[1], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[1] == true) {
      SensorValue[1] = analogRead(Sensor[1]);
      Samples1.add(SensorValue[1]);
      delay(Wait);
      SensorFinal[1] = Samples1.getMedian();
      if (SensorFinal[1] < Thresh) {
        usbMIDI.sendNoteOff(Note[1], 0, 1);
        usbMIDI.sendNoteOff(Note[1] - AddNote, 0, 1);
        NotePlayed[1] = false;        
      }
      if (TrigAddon[1] == true) {
        TrigAddon[1] = false;
        Note[1] = Note[1] + AddNote;
      }
    }
  }
  if (SustainOn == false && NotePlayed[1]  == false) {
    digitalWrite(RedLed[1], LOW);
    digitalWrite(BlueLed[1], LOW);
  }

  // ********************sensor 0*******************
  if (NotePlayed[0] == false) {
    SensorValue[0] = analogRead(Sensor[0]);
    Samples0.add(SensorValue[0]);
    delay(Wait);
    SensorFinal[0] = Samples0.getMedian();
    if (SensorFinal[0] > ThreshHigh) {
      Note[0] = Note[0] - AddNote;
      TrigAddon[0] = true;
    }
    if (SensorFinal[0] > Thresh) {
      usbMIDI.sendNoteOn(Note[0], 255, 1);
      NotePlayed[0] = true;
      if (TrigAddon[0] == true) {
        digitalWrite(RedLed[0], HIGH);
      }
      else {
        digitalWrite(BlueLed[0], HIGH);
      }
    }
  }

  else {
    if (NotePlayed[0] == true) {
      SensorValue[0] = analogRead(Sensor[0]);
      Samples0.add(SensorValue[0]);
      delay(Wait);
      SensorFinal[0] = Samples0.getMedian();
      if (SensorFinal[0] < Thresh) {
        usbMIDI.sendNoteOff(Note[0], 0, 1);
        usbMIDI.sendNoteOff(Note[0] - AddNote, 0, 1);
        NotePlayed[0] = false;        
      }
      if (TrigAddon[0] == true) {
        TrigAddon[0] = false;
        Note[0] = Note[0] + AddNote;
      }
    }
  }
  if (SustainOn == false && NotePlayed[0]  == false) {
    digitalWrite(RedLed[0], LOW);
    digitalWrite(BlueLed[0], LOW);
  }

}


//********************************SUBROUTINES**************************

void CleanDisplay() {
  if (Note[m] < 100 && Note[m] > 10) {
    lcd.setCursor(11, 1); lcd.print("     ");
  }

  if (Note[m] < 10) {
    lcd.setCursor(10, 1); lcd.print("      ");
  }
}

//*********************************************************************
void CheckNote() {

  if (digitalRead(NoteSelectButton) == LOW) {
    Check = true; m = 0;
    lcd.setCursor(0, 0); lcd.print("  *SET NOTES*   ");
    lcd.setCursor(0, 1); lcd.print("note:           ");
    delay(300);
  }

  while (Check == true) {

    lcd.setCursor(6, 1); lcd.print(m + 1);
    lcd.setCursor(9, 1); lcd.print(Note[m]); CleanDisplay();
    if (digitalRead(upEncoderPin) == LOW && digitalRead(downEncoderPin) == LOW)
    {
      Note[m] = Note[m] - 1; delay(100);
      if (Note[m] < 0) {
        Note[m] = 0;
      }
      lcd.setCursor(9, 1); lcd.print(Note[m]); CleanDisplay();
    }

    else if (digitalRead(upEncoderPin) == LOW && digitalRead(downEncoderPin) == HIGH)

    {
      Note[m] = Note[m] + 1; delay(100);
      if (Note[m] > 127) {
        Note[m] = 127;
      }
      lcd.setCursor(9, 1); lcd.print(Note[m]); CleanDisplay();
    }

    if (digitalRead(NoteSelectButton) == LOW) {
      delay(500);
      lcd.setCursor(9, 1); lcd.print(Note[m]); CleanDisplay();
      m = m + 1;
    }

    if (digitalRead(OctaveUpButton) == LOW) {

      lcd.setCursor(0, 0); lcd.print(" *RESET NOTES*  ");
      lcd.setCursor(0, 1); lcd.print("                ");
      Note[0] = 64;
      Note[1] = 66;
      Note[2] = 68;
      Note[3] = 69;
      Note[4] = 71;
      Note[5] = 73;
      Note[6] = 75;
      Note[7] = 76;
      delay(3000);
      lcd.setCursor(0, 0); lcd.print("     oct        ");
      Select();
      Check = false;
    }

    if (m == 8) {
      m = 0;
      Check = false;
      lcd.setCursor(0, 0); lcd.print("NOTES PROGRAMMED");
      lcd.setCursor(0, 1); lcd.print("                ");
      delay(3000);
      lcd.setCursor(0, 0); lcd.print("     oct        ");
      Select();

    }
  }
}

//*********************************************************************
void CheckOctave() {

  if (digitalRead(OctaveDownButton) == LOW) {
    delay(300);
    if (Octave > 0) {
      Octave = Octave - 1;
      for (k = 0; k < SensorNumber; k++) {
        usbMIDI.sendNoteOff(Note[k], 0, 1); Note[k] = Note[k] - AddNote;
      }
      lcd.setCursor(8, 0); lcd.print(OctString[Octave]);
    }
  }

  if (digitalRead(OctaveUpButton) == LOW) {
    delay(300);
    if (Octave < 4) {
      Octave = Octave + 1;
      for (k = 0; k < SensorNumber; k++) {
        usbMIDI.sendNoteOff(Note[k], 0, 1); Note[k] = Note[k] + AddNote;
      }
      lcd.setCursor(8, 0); lcd.print(OctString[Octave]);
    }
  }
}

//*********************************************************************
void CheckInstrument()
{
  if (digitalRead(upEncoderPin) == LOW && digitalRead(downEncoderPin) == LOW)
  {
    j = j - 1; delay(100);
    Select();
  }
  else if (digitalRead(upEncoderPin) == LOW && digitalRead(downEncoderPin) == HIGH)

  {
    j = j + 1; delay(100);
    Select();
  }
}

//*********************************************************************
void Select()
{
  //this is the list of instruments, new instruments can be added
  if (j < 1  ) {
    j = 1;
  }
  if (j == 57 ) {
    j = 56;
  }

  switch (j) {

    case 1:
      lcd.setCursor(0, 1);
      lcd.print("grand piano     ");
      usbMIDI.sendProgramChange(0, 1);
      break;

    case 2:
      lcd.setCursor(0, 1);
      lcd.print("honky-tonk piano");
      usbMIDI.sendProgramChange(3, 1);
      break;

    case 3:
      lcd.setCursor(0, 1);
      lcd.print("harpsichord     ");
      usbMIDI.sendProgramChange(6, 1);
      break;

    case 4:
      lcd.setCursor(0, 1);
      lcd.print("glockenspiel    ");
      usbMIDI.sendProgramChange(9, 1);
      break;

    case 5:
      lcd.setCursor(0, 1);
      lcd.print("music box       ");
      usbMIDI.sendProgramChange(10, 1);
      break;

    case 6:
      lcd.setCursor(0, 1);
      lcd.print("vibraphone      ");
      usbMIDI.sendProgramChange(11, 1);
      break;

    case 7:
      lcd.setCursor(0, 1);
      lcd.print("xylophone       ");
      usbMIDI.sendProgramChange(13, 1);
      break;

    case 8:
      lcd.setCursor(0, 1);
      lcd.print("tubular bells   ");
      usbMIDI.sendProgramChange(14, 1);
      break;

    case 9:
      lcd.setCursor(0, 1);
      lcd.print("dulcimer       ");
      usbMIDI.sendProgramChange(15, 1);
      break;

    case 10:
      lcd.setCursor(0, 1);
      lcd.print("church organ    ");
      usbMIDI.sendProgramChange(19, 1);
      break;

    case 11:
      lcd.setCursor(0, 1);
      lcd.print("accordion       ");
      usbMIDI.sendProgramChange(21, 1);
      break;

    case 12:
      lcd.setCursor(0, 1);
      lcd.print("harmonica       ");
      usbMIDI.sendProgramChange(22, 1);
      break;

    case 13:
      lcd.setCursor(0, 1);
      lcd.print("tango accordion ");
      usbMIDI.sendProgramChange(23, 1);
      break;

    case 14:
      lcd.setCursor(0, 1);
      lcd.print("clean el. guitar");
      usbMIDI.sendProgramChange(27, 1);
      break;

    case 15:
      lcd.setCursor(0, 1);
      lcd.print("slap bass1      ");
      usbMIDI.sendProgramChange(36, 1);
      break;

    case 16:
      lcd.setCursor(0, 1);
      lcd.print("synth bass 2    ");
      usbMIDI.sendProgramChange(39, 1);
      break;

    case 17:
      lcd.setCursor(0, 1);
      lcd.print("violin          ");
      usbMIDI.sendProgramChange(40, 1);
      break;

    case 18:
      lcd.setCursor(0, 1);
      lcd.print("tremolo strings ");
      usbMIDI.sendProgramChange(44, 1);
      break;

    case 19:
      lcd.setCursor(0, 1);
      lcd.print("pizzicato string");
      usbMIDI.sendProgramChange(45, 1);
      break;

    case 20:
      lcd.setCursor(0, 1);
      lcd.print("orchestral harp ");
      usbMIDI.sendProgramChange(46, 1);
      break;

    case 21:
      lcd.setCursor(0, 1);
      lcd.print("synth strings 1 ");
      usbMIDI.sendProgramChange(50, 1);
      break;

    case 22:
      lcd.setCursor(0, 1);
      lcd.print("choir aahs      ");
      usbMIDI.sendProgramChange(52, 1);
      break;

    case 23:
      lcd.setCursor(0, 1);
      lcd.print("voice oohs      ");
      usbMIDI.sendProgramChange(53, 1);
      break;

    case 24:
      lcd.setCursor(0, 1);
      lcd.print("orchestra hit   ");
      usbMIDI.sendProgramChange(55, 1);
      break;

    case 25:
      lcd.setCursor(0, 1);
      lcd.print("trombone        ");
      usbMIDI.sendProgramChange(57, 1);
      break;

    case 26:
      lcd.setCursor(0, 1);
      lcd.print("french horn     ");
      usbMIDI.sendProgramChange(60, 1);
      break;
    case 27:
      lcd.setCursor(0, 1);
      lcd.print("alto sax        ");
      usbMIDI.sendProgramChange(65, 1);
      break;

    case 28:
      lcd.setCursor(0, 1);
      lcd.print("bariton sax     ");
      usbMIDI.sendProgramChange(67, 1);
      break;

    case 29:
      lcd.setCursor(0, 1);
      lcd.print("clarinet        ");
      usbMIDI.sendProgramChange(71, 1);
      break;

    case 30:
      lcd.setCursor(0, 1);
      lcd.print("flute           ");
      usbMIDI.sendProgramChange(73, 1);
      break;

    case 31:
      lcd.setCursor(0, 1);
      lcd.print("pan flute       ");
      usbMIDI.sendProgramChange(75, 1);
      break;

    case 32:
      lcd.setCursor(0, 1);
      lcd.print("blown bottle    ");
      usbMIDI.sendProgramChange(76, 1);
      break;

    case 33:
      lcd.setCursor(0, 1);
      lcd.print("sawtooth        ");
      usbMIDI.sendProgramChange(81, 1);
      break;

    case 34:
      lcd.setCursor(0, 1);
      lcd.print("fifths          ");
      usbMIDI.sendProgramChange(86, 1);
      break;

    case 35:
      lcd.setCursor(0, 1);
      lcd.print("bass + lead     ");
      usbMIDI.sendProgramChange(87, 1);
      break;

    case 36:
      lcd.setCursor(0, 1);
      lcd.print("new age         ");
      usbMIDI.sendProgramChange(88, 1);
      break;

    case 37:
      lcd.setCursor(0, 1);
      lcd.print("bowed           ");
      usbMIDI.sendProgramChange(92, 1);
      break;

    case 38:
      lcd.setCursor(0, 1);
      lcd.print("metallic        ");
      usbMIDI.sendProgramChange(93, 1);
      break;

    case 39:
      lcd.setCursor(0, 1);
      lcd.print("ice rain        ");
      usbMIDI.sendProgramChange(96, 1);
      break;

    case 40:
      lcd.setCursor(0, 1);
      lcd.print("soundtrack      ");
      usbMIDI.sendProgramChange(97, 1);
      break;

    case 41:
      lcd.setCursor(0, 1);
      lcd.print("crystal         ");
      usbMIDI.sendProgramChange(98, 1);
      break;

    case 42:
      lcd.setCursor(0, 1);
      lcd.print("echoes          ");
      usbMIDI.sendProgramChange(102, 1);
      break;

    case 43:
      lcd.setCursor(0, 1);
      lcd.print("sci-fi          ");
      usbMIDI.sendProgramChange(103, 1);
      break;

    case 44:
      lcd.setCursor(0, 1);
      lcd.print("sitar           ");
      usbMIDI.sendProgramChange(104, 1);
      break;

    case 45:
      lcd.setCursor(0, 1);
      lcd.print("banjo           ");
      usbMIDI.sendProgramChange(105, 1);
      break;

    case 46:
      lcd.setCursor(0, 1);
      lcd.print("shamisen        ");
      usbMIDI.sendProgramChange(106, 1);
      break;

    case 47:
      lcd.setCursor(0, 1);
      lcd.print("koto            ");
      usbMIDI.sendProgramChange(107, 1);
      break;

    case 48:
      lcd.setCursor(0, 1);
      lcd.print("kalimba         ");
      usbMIDI.sendProgramChange(108, 1);
      break;

    case 49:
      lcd.setCursor(0, 1);
      lcd.print("shanai          ");
      usbMIDI.sendProgramChange(111, 1);
      break;

    case 50:
      lcd.setCursor(0, 1);
      lcd.print("tinkle bell     ");
      usbMIDI.sendProgramChange(112, 1);
      break;

    case 51:
      lcd.setCursor(0, 1);
      lcd.print("agogo           ");
      usbMIDI.sendProgramChange(113, 1);
      break;

    case 52:
      lcd.setCursor(0, 1);
      lcd.print("steel drums     ");
      usbMIDI.sendProgramChange(114, 1);
      break;

    case 53:
      lcd.setCursor(0, 1);
      lcd.print("taiko drum      ");
      usbMIDI.sendProgramChange(116, 1);
      break;

    case 54:
      lcd.setCursor(0, 1);
      lcd.print("guitarfret noise");
      usbMIDI.sendProgramChange(120, 1);
      break;

    case 55:
      lcd.setCursor(0, 1);
      lcd.print("seashore        ");
      usbMIDI.sendProgramChange(122, 1);
      break;

    case 56:
      lcd.setCursor(0, 1);
      lcd.print("bird tweet      ");
      usbMIDI.sendProgramChange(123, 1);
      break;

    default:
      break;

  }
}

//***********************************************************
void CheckSustain()
{
  //switch sustain ON and show this on the display
  if (digitalRead(SustainSwitch) == LOW ) {
    usbMIDI.sendControlChange(64, 64, 1);
    SustainOn = true;
    lcd.setCursor(12, 0);
    lcd.print("sust");
  }
  //switch sustain OFF and show this on the display
  if (digitalRead(SustainSwitch) == HIGH ) {
    usbMIDI.sendControlChange(64, 0, 1);
    SustainOn = false;
    lcd.setCursor(12, 0);
    lcd.print("    ");

  }

}

//********************************************************
void CheckVibrato()
{
  //switch vibrato ON and show this on the display
  if (digitalRead(VibratoSwitch) == LOW ) {
    usbMIDI.sendControlChange(1, 127, 1);
    lcd.setCursor(0, 0);
    lcd.print("vibr");
  }
  //switch vibrato OFF and show this on the display
  if (digitalRead(VibratoSwitch) == HIGH ) {
    usbMIDI.sendControlChange(1, 0, 1);
    lcd.setCursor(0, 0);
    lcd.print("    ");

  }

}

//********************************************************
void CheckPitchBend()
{
  PitchBendSample = analogRead(PitchBendPin);  // 0 to 1023 in value
  PitchValue = 0;
  if (PitchBendSample < LowerTrigger) {
    PitchValue = (LowerTrigger - PitchBendSample) * (-50);
  }
  if (PitchBendSample > HigherTrigger) {
    PitchValue = (PitchBendSample - HigherTrigger) * 55;
  }
  if (PitchValue > 8192) {
    PitchValue = 8192;
  }
  if (PitchValue < -8192) {
    PitchValue = -8192;
  }
  usbMIDI.sendPitchBend(PitchValue, 1);

}
